// BasicShader.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "cube.h"
#include "sphere.h"
#include "banana.h"
#include <GL/glew.h>
#include <iostream>
#include <GL/glut.h>
#include <SDL/SDL.h> //NEW
#include <SDL/SDL_image.h> // NEW
#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>
#include <glm/gtc/matrix_inverse.hpp>
#include <glm/gtc/type_ptr.hpp>
#include <string.h>
#include <fstream>
#include <sstream>

// using namespace std;

GLfloat rotace = 0.0;

GLuint programObject;

glm::mat4 projectionMatrix;
glm::mat4 modelMatrix;
glm::mat4 viewMatrix;

glm::mat4 modelViewMatrix; //
glm::mat4 MVP; //
glm::mat3 normalMatrix; //

GLuint VAOHandle[3];

GLfloat mouseX=0, mouseY=0;
bool mouseLeftDown=false, mouseRightDown=false;

GLfloat cameraAngleX=0, cameraAngleY=0;
GLfloat cameraDistance=0;



bool LoadTXTFile(const char *fileName, std::string &textFile)
	{
		std::stringstream strStream;

		std::ifstream file(fileName, std::ifstream::in);
		if(!file.good())
			return false;

		strStream << file.rdbuf();
		textFile = strStream.str();
		file.close();

		return true;
	}

void InitGlut(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(500, 500);
	glutInitWindowPosition(100, 100);
	glutCreateWindow(argv[0]);
}

void objectLoad(const int VAOH,
				const int numVertices,
				const float vertices[],
				const float normals[],
				const float texCoords[])
{
	//VBO
	GLuint VBOHandle[3];
	glGenBuffers(3, VBOHandle);

	GLuint positionBufferHandle= VBOHandle[0];
	GLuint normalBufferHandle= VBOHandle[1];
	GLuint texCoordBufferHandle= VBOHandle[2];

	if(vertices)
	{
		glBindBuffer(GL_ARRAY_BUFFER, positionBufferHandle);
		glBufferData(GL_ARRAY_BUFFER, 3*numVertices*sizeof(float), vertices, GL_STATIC_DRAW);
	}

	if(normals)
	{
		glBindBuffer(GL_ARRAY_BUFFER, normalBufferHandle);
		glBufferData(GL_ARRAY_BUFFER, 3*numVertices*sizeof(float), normals, GL_STATIC_DRAW);
	}

	if(texCoords)
	{
		glBindBuffer(GL_ARRAY_BUFFER, texCoordBufferHandle);
		glBufferData(GL_ARRAY_BUFFER, 3*numVertices*sizeof(float), texCoords, GL_STATIC_DRAW);
	}

	const GLint position = 0;
	const GLint norm = 1;
	const GLint texc = 2;

	//VAO
	glBindVertexArray(VAOH);

	// Enable Vertex Attribute Array
	glEnableVertexAttribArray(position);
	glEnableVertexAttribArray(norm);
	glEnableVertexAttribArray(texc);

	// Prirad index nula na positionBuffer
	glBindBuffer(GL_ARRAY_BUFFER, positionBufferHandle);
	glVertexAttribPointer(position, 3, GL_FLOAT, GL_FALSE, 0, (GLubyte *) NULL);

	// Prirad index jedna na normalBuffer
	glBindBuffer(GL_ARRAY_BUFFER, normalBufferHandle);
	glVertexAttribPointer(norm, 3, GL_FLOAT, GL_FALSE, 0, (GLubyte *) NULL);

	// Prirad index dve na texCoordBuffer
	glBindBuffer(GL_ARRAY_BUFFER, texCoordBufferHandle);
	glVertexAttribPointer(texc, 3, GL_FLOAT, GL_FALSE, 0, (GLubyte *) NULL);
}

void GLInit()
{
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glShadeModel(GL_FLAT);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D); //NEW

	glGenVertexArrays(3, VAOHandle);

	objectLoad(VAOHandle[0], cubeNumVerts, cubeVerts, cubeNormals, (GLfloat *) NULL);
	objectLoad(VAOHandle[1], SphereNumVerts, SphereVerts, SphereNormals, SphereTexCoords);
	objectLoad(VAOHandle[2], bananaNumVerts, bananaVerts, bananaNormals, bananaTexCoords);
}

void transformations(void)
{
	//glLoadIdentity();
	viewMatrix= glm::mat4(1.0f);
	//gluLookAt(0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
	glm::vec3 eye(0.0f, 0.0f, 5.0f);
	glm::vec3 center(0.0f, 0.0f, 0.0f);
	glm::vec3 up(0.0f, 1.0f, 0.0f);
	viewMatrix= glm::lookAt(eye, center, up);

	//identita
	modelMatrix= glm::mat4(1.0f);
	//glTranslatef(0, 0, cameraDistance);
	modelMatrix= glm::translate(modelMatrix, glm::vec3(0.0f, 0.0f, cameraDistance));
	//glRotatef(cameraAngleX, 1.0, 0.0, 0.0);
	modelMatrix= glm::rotate(modelMatrix, cameraAngleX, glm::vec3(1.0f, 0.0f, 0.0f));
	//glRotatef(cameraAngleY, 0.0, 1.0, 0.0);
	modelMatrix= glm::rotate(modelMatrix, cameraAngleY, glm::vec3(0.0f, 1.0f, 0.0f));
	//glRotatef(-90, 1.0, 0.0, 0.0);
	modelMatrix= glm::rotate(modelMatrix, -90.0f, glm::vec3(1.0f, 0.0f, 0.0f));

	modelViewMatrix= viewMatrix * modelMatrix;
	MVP= projectionMatrix * viewMatrix * modelMatrix;
	// leva horni 3x3 submatice z MVP, transponovana inverze
	normalMatrix= glm::mat3(MVP);
	normalMatrix= glm::inverseTranspose(normalMatrix);

	GLint MVPLoc= glGetUniformLocation(programObject, "MVP");
	glUniformMatrix4fv(MVPLoc, 1, GL_FALSE, glm::value_ptr(MVP));

	GLint normalMatrixLoc= glGetUniformLocation(programObject, "normalMatrix");
	glUniformMatrix3fv(normalMatrixLoc, 1, GL_FALSE, glm::value_ptr(normalMatrix));

	GLint modelViewMatrixLoc= glGetUniformLocation(programObject, "modelViewMatrix");
	glUniformMatrix4fv(modelViewMatrixLoc, 1, GL_FALSE, glm::value_ptr(modelViewMatrix));

}

void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT |  GL_DEPTH_BUFFER_BIT);

	transformations();

	// Prepnuti VAO
	glBindVertexArray(VAOHandle[1]);
	glDrawArrays(GL_TRIANGLES, 0, (GLsizei) cubeNumVerts);

	//glBegin(GL_TRIANGLES);
	//  glVertexAttrib3f(position, 1.0, 1.0, 0.0);
	//  glVertexAttrib3f(norm, 0.0, 0.0, 1.0);

	//  glVertexAttrib3f(position, 0.0, 1.0, 0.0);
	//  glVertexAttrib3f(norm, 0.0, 0.0, 1.0);

	//  glVertexAttrib3f(position, 1.0, 0.0, 0.0);
	//  glVertexAttrib3f(norm, 0.0, 0.0, 1.0);
	//glEnd();

	glFlush();
	glutSwapBuffers();
}

void reshape(int w, int h)
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	//glMatrixMode(GL_PROJECTION);
	//glLoadIdentity();
	projectionMatrix= glm::mat4(1.0f);
	        //(Left, Right, Bottom, Top, Near, Far);
	//glFrustum(-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);
	projectionMatrix= glm::frustum(-1.0f, 1.0f, -1.0f, 1.0f, 1.5f, 20.0f);
	//glMatrixMode(GL_MODELVIEW);
}

void mouse(int button, int state, int x, int y)
{
    mouseX = x;
    mouseY = y;

    if(button == GLUT_LEFT_BUTTON)
    {
        if(state == GLUT_DOWN)
        {
            mouseLeftDown = true;
        }
        else if(state == GLUT_UP)
            mouseLeftDown = false;
    }

    else if(button == GLUT_RIGHT_BUTTON)
    {
        if(state == GLUT_DOWN)
        {
            mouseRightDown = true;
        }
        else if(state == GLUT_UP)
            mouseRightDown = false;
    }
}

void mouseMotion(int x, int y)
{
    if(mouseLeftDown)
    {
        cameraAngleY += (x - mouseX);
        cameraAngleX += (y - mouseY);
        mouseX = x;
        mouseY = y;
    }
    if(mouseRightDown)
    {
        cameraDistance += (y - mouseY) * 0.2f;
        mouseY = y;
    }
	glutPostRedisplay();
}

void keyboard(unsigned char key, int x, int y)
{
	if ((int)key == 27) exit(0);
	if (key == 'x') rotace += 1.0;
	if (key == 'z') rotace -= 1.0;
	glutPostRedisplay();
}

void GlutCallbacks()
{
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouse);
    glutMotionFunc(mouseMotion);

}

void CheckVersions()
{
	char * version, * vendor, * renderer, * shadingLanguage;

	version = (char*)glGetString(GL_VERSION);
	vendor = (char*)glGetString(GL_VENDOR);
	renderer = (char*)glGetString(GL_RENDERER);
	shadingLanguage = (char*)glGetString(GL_SHADING_LANGUAGE_VERSION);

	std::cout << "Verze OpenGL:" << version << "\n";
	std::cout << "Vyrobce karty:" << vendor << "\n";
	std::cout << "Karta:" << renderer << "\n";
	std::cout << "Verze GLSL:" << shadingLanguage << "\n\n";
}

void CreateFirstShader()
{
	GLint status;
	// 1. Vytvoreni objektu shaderu, ulozeni id
	GLuint vertexShaderObject;
	GLuint fragmentShaderObject;
	vertexShaderObject = glCreateShader(GL_VERTEX_SHADER);
	fragmentShaderObject = glCreateShader(GL_FRAGMENT_SHADER);

	// 2. Nahrati zdrojoveho kodu ze souboru
	std::string vertexShaderTextString;
	std::string fragmentShaderTextString;
	const char * vertexShaderName = "VertexShader.sh";
	const char * fragmentShaderName = "FragmentShader.sh";

	if(LoadTXTFile(vertexShaderName, vertexShaderTextString))
		std::cout << vertexShaderName << "loaded\n";
	else std::cout << vertexShaderName << "loading ERROR!!!\n";

	if(LoadTXTFile(fragmentShaderName, fragmentShaderTextString))
		std::cout << fragmentShaderName << "loaded\n";
	else std::cout << fragmentShaderName << "loading ERROR!!!\n";

	const char * vs = vertexShaderTextString.c_str();
	const char * fs = fragmentShaderTextString.c_str();
	glShaderSource(vertexShaderObject, 1, &vs, NULL);
	glShaderSource(fragmentShaderObject, 1, &fs, NULL);

	// 3. Kompilace shaderu
	glCompileShader(vertexShaderObject);
	glGetShaderiv(vertexShaderObject, GL_COMPILE_STATUS, &status);
	if(status) std::cout << "Vertex shader compilation OK\n";
	else
	{
		std::cout << "Vertex shader compilation ERROR!!!\n";
		GLchar infoLog[5000];
		glGetShaderInfoLog(vertexShaderObject, 5000, NULL, infoLog);
		std::cout << infoLog << "\n";
	}

	glCompileShader(fragmentShaderObject);
	glGetShaderiv(fragmentShaderObject, GL_COMPILE_STATUS, &status);
	if(status) std::cout << "Fragment shader compilation OK\n";
	else
	{
		std::cout << "Fragment shader compilation ERROR!!!\n";
		GLchar infoLog[5000];
		glGetShaderInfoLog(fragmentShaderObject, 5000, NULL, infoLog);
		std::cout << infoLog << "\n";
	}

	// 4. Vytvoreni objektu programu
	programObject= glCreateProgram();

	// 5. Pripojeni objektu shaderu k programovemu objektu
	glAttachShader(programObject, vertexShaderObject);
	glAttachShader(programObject, fragmentShaderObject);

	// 6. Slinkovani programu
	glLinkProgram(programObject);
	glGetProgramiv(programObject, GL_LINK_STATUS, &status);
	if(status) std::cout << "Linking OK\n";
	else
	{
		std::cout << "Linking ERROR!!!\n";
		GLchar infoLog[5000];
		glGetProgramInfoLog(programObject, 5000, NULL, infoLog);
		std::cout << infoLog << "\n";
	}

	// 7. Pouziti programu
	glUseProgram(programObject);

	// Zjisteni lokace uniformni promenne
	GLint lightPositionLoc;
	lightPositionLoc = glGetUniformLocation(programObject, "lightPosition");
	glUniform3f(lightPositionLoc, 0.0f, 0.0f, 4.0f);

	GLint brickSizeLoc;
	brickSizeLoc = glGetUniformLocation(programObject, "brickSize");
	glUniform2f(brickSizeLoc, 0.3f, 0.15f);

	GLint brickPercentLoc;
	brickPercentLoc = glGetUniformLocation(programObject, "brickPercent");
	glUniform2f(brickPercentLoc, 0.9f, 0.85f);



}

int _tmain(int argc, char** argv)
{
	InitGlut(argc, argv);
	GLInit();
	glewInit();

	CheckVersions();

	GlutCallbacks();

	CreateFirstShader();

	glutMainLoop();
	return 0;
}

